"use strict";

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { "default": obj }; };

var _classCallCheck = function (instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(object, property, receiver) { var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _inherits = function (subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) subClass.__proto__ = superClass; };

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _EventEmitter2 = require("events");

var _dirname = require("path");

var _unlink$unlinkSync$rmdir$rmdirSync = require("fs");

var _mkdir$mkdirSync = require("mkdirp");

var _mkdir$mkdirSync2 = _interopRequireWildcard(_mkdir$mkdirSync);

var _Minimatch = require("minimatch");

var _Glob$searchSync = require("glob");

var _getBasePath = require("glob2base");

var _getBasePath2 = _interopRequireWildcard(_getBasePath);

var _createWatcher = require("chokidar");

var _import = require("./copy");

var cp = _interopRequireWildcard(_import);

var _assert$assertType$assertTypeOpt = require("./utils");

var _Queue = require("./queue");

var _Queue2 = _interopRequireWildcard(_Queue);

var SOURCE = Symbol("source");
var OUT_DIR = Symbol("outDir");
var TRANSFORM = Symbol("transform");
var QUEUE = Symbol("queue");
var WATCHER = Symbol("watcher");

// Glob is not supported delimiters of Windows.
function normalizePath(path) {
  if (path == null) {
    return null;
  }
  path = path.replace(/\\/g, "/").trim();
  if (/\/$/.test(path)) {
    path = path.slice(0, -1);
  }
  return path;
}

// Call the action for every files that matches the pattern.
function doAllSimply(cpx, pattern, action) {
  new _Glob$searchSync.Glob(pattern, { nodir: true, silent: true }).on("match", action.bind(cpx));
}

// Call the action for every files that matches the pattern.
function doAll(cpx, pattern, action, cb) {
  if (cb == null) {
    doAllSimply(cpx, pattern, action);
    return;
  }

  var count = 0;
  var done = false;
  var lastError = null;
  var cbIfEnd = function cbIfEnd() {
    if (done && count === 0) {
      cb(lastError);
    }
  };

  new _Glob$searchSync.Glob(pattern, { nodir: true, silent: true }).on("match", function (path) {
    if (lastError != null) {
      return;
    }

    count += 1;
    action.call(cpx, path, function (err) {
      count -= 1;
      lastError = lastError || err;
      cbIfEnd();
    });
  }).on("end", function () {
    done = true;
    cbIfEnd();
  }).on("error", function (err) {
    lastError = lastError || err;
  });
}

var Cpx = (function (_EventEmitter) {
  /**
   * @param {string} source - A blob for copy files.
   * @param {string} outDir - A file path for the destination directory.
   */

  function Cpx(source, outDir, options) {
    _classCallCheck(this, Cpx);

    _assert$assertType$assertTypeOpt.assertType(source, "source", "string");
    _assert$assertType$assertTypeOpt.assertType(outDir, "outDir", "string");
    _get(Object.getPrototypeOf(Cpx.prototype), "constructor", this).call(this);

    var transforms = [].concat(options && options.transform).filter(Boolean);
    transforms.forEach(function (t) {
      return _assert$assertType$assertTypeOpt.assertType(t, "transform", "function");
    });

    this[SOURCE] = normalizePath(source);
    this[OUT_DIR] = normalizePath(outDir);
    this[TRANSFORM] = transforms;
    this[QUEUE] = new _Queue2["default"]();
    this[WATCHER] = null;
  }

  _inherits(Cpx, _EventEmitter);

  _createClass(Cpx, [{
    key: "source",

    //============================================================================
    // Commons
    //----------------------------------------------------------------------------

    /**
     * The source file glob to copy.
     * @type {string}
     */
    get: function () {
      return this[SOURCE];
    }
  }, {
    key: "outDir",

    /**
     * The destination directory to copy.
     * @type {string}
     */
    get: function () {
      return this[OUT_DIR];
    }
  }, {
    key: "transformFactories",

    /**
     * The factories of transform streams.
     * @type {function[]}
     */
    get: function () {
      return this[TRANSFORM];
    }
  }, {
    key: "base",

    /**
     * The base directory of `this.source`.
     * @type {string}
     */
    get: function () {
      var value = normalizePath(_getBasePath2["default"](new _Glob$searchSync.Glob(this.source)));
      Object.defineProperty(this, "base", { value: value, configurable: true });
      return value;
    }
  }, {
    key: "src2dst",

    /**
     * Convert a glob from source to destination.
     * @param {string} path
     * @returns {string}
     */
    value: function src2dst(path) {
      _assert$assertType$assertTypeOpt.assertType(path, "path", "string");
      return path.replace(this.base, this.outDir);
    }
  }, {
    key: "enqueueCopy",

    /**
     * Copy a file sequentially.
     * @param {string} srcPath
     * @param {cpx~callback} [cb = null]
     */
    value: function enqueueCopy(srcPath) {
      var _this = this;

      var cb = arguments[1] === undefined ? null : arguments[1];

      _assert$assertType$assertTypeOpt.assertType(srcPath, "srcPath", "string");
      _assert$assertType$assertTypeOpt.assertTypeOpt(cb, "cb", "function");

      var dstPath = this.src2dst(srcPath);
      if (dstPath === srcPath) {
        if (cb != null) {
          setImmediate(cb, null);
          return;
        }
      }

      this[QUEUE].push(function (next) {
        _mkdir$mkdirSync2["default"](_dirname.dirname(dstPath), next);
      });
      this[QUEUE].push(function (next) {
        cp.copy(srcPath, dstPath, _this.transformFactories, function (err) {
          if (err == null) {
            _this.emit("copy", { srcPath: srcPath, dstPath: dstPath });
          }

          next();
          if (cb != null) {
            cb(err || null);
          }
        });
      });
    }
  }, {
    key: "enqueueRemove",

    /**
     * Remove a file sequentially.
     * @param {string} path
     * @param {cpx~callback} [cb = null]
     */
    value: function enqueueRemove(path) {
      var _this2 = this;

      var cb = arguments[1] === undefined ? null : arguments[1];

      _assert$assertType$assertTypeOpt.assertType(path, "path", "string");
      _assert$assertType$assertTypeOpt.assertTypeOpt(cb, "cb", "function");

      var lastError = null;
      this[QUEUE].push(function (next) {
        _unlink$unlinkSync$rmdir$rmdirSync.unlink(path, function (err) {
          if (err == null) {
            _this2.emit("remove", { path: path });
          }

          lastError = err;
          next();
        });
      });
      this[QUEUE].push(function (next) {
        _unlink$unlinkSync$rmdir$rmdirSync.rmdir(_dirname.dirname(path), function () {
          next();
          if (cb != null) {
            cb(lastError);
          }
        });
      });
    }
  }, {
    key: "clean",

    //============================================================================
    // Clean Methods
    //----------------------------------------------------------------------------

    /**
     * Remove all files that matches `this.source` like pattern in `this.dest`
     * directory.
     * @param {cpx~callback} [cb = null]
     */
    value: function clean() {
      var cb = arguments[0] === undefined ? null : arguments[0];

      _assert$assertType$assertTypeOpt.assertTypeOpt(cb, "cb", "function");

      var dest = this.src2dst(this.source);
      if (dest === this.source) {
        if (cb != null) {
          setImmediate(cb, null);
        }
        return;
      }

      doAll(this, dest, this.enqueueRemove, cb);
    }
  }, {
    key: "cleanSync",

    /**
     * Remove all files that matches `this.source` like pattern in `this.dest`
     * directory.
     * @thrpws {Error} IO error.
     */
    value: function cleanSync() {
      var _this3 = this;

      var dest = this.src2dst(this.source);
      if (dest === this.source) {
        return;
      }

      var pathes = _Glob$searchSync.sync(dest, { nodir: true, silent: true });
      pathes.forEach(function (path) {
        _unlink$unlinkSync$rmdir$rmdirSync.unlinkSync(path);
        try {
          _unlink$unlinkSync$rmdir$rmdirSync.rmdirSync(_dirname.dirname(path));
        } catch (err) {
          if (err.code !== "ENOTEMPTY") {
            throw err;
          }
        }
        _this3.emit("remove", { path: path });
      });
    }
  }, {
    key: "copy",

    //============================================================================
    // Copy Methods
    //----------------------------------------------------------------------------

    /**
     * Copy all files that matches `this.source` pattern to `this.outDir`.
     * @param {cpx~callback} [cb = null]
     */
    value: function copy() {
      var cb = arguments[0] === undefined ? null : arguments[0];

      _assert$assertType$assertTypeOpt.assertTypeOpt(cb, "cb", "function");
      doAll(this, this.source, this.enqueueCopy, cb);
    }
  }, {
    key: "copySync",

    /**
     * Copy all files that matches `this.source` pattern to `this.outDir`.
     * @thrpws {Error} IO error.
     */
    value: function copySync() {
      var _this4 = this;

      _assert$assertType$assertTypeOpt.assert(this.transformFactories.length === 0, "Synchronous copy can't use the transform option.");

      var srcPathes = _Glob$searchSync.sync(this.source, { nodir: true, silent: true });
      srcPathes.forEach(function (srcPath) {
        var dstPath = _this4.src2dst(srcPath);
        if (dstPath === srcPath) {
          return; //continue
        }

        _mkdir$mkdirSync.sync(_dirname.dirname(dstPath));
        cp.copySync(srcPath, dstPath);

        _this4.emit("copy", { srcPath: srcPath, dstPath: dstPath });
      });
    }
  }, {
    key: "watch",

    //============================================================================
    // Watch Methods
    //----------------------------------------------------------------------------

    /**
     * Copy all files that matches `this.source` pattern to `this.outDir`.
     * And watch changes in `this.base`, and copy only the file every time.
     * @throws {Error} This had been watching already.
     */
    value: function watch() {
      var _this5 = this;

      if (this[WATCHER] != null) {
        throw new Error("InvalidStateError");
      }

      var m = new _Minimatch.Minimatch(this.source);
      var firstCopyCount = 0;
      var ready = false;
      var fireReadyIfReady = function fireReadyIfReady() {
        if (ready && firstCopyCount === 0) {
          _this5.emit("watch-ready");
        }
      };

      this[WATCHER] = _createWatcher.watch(this.base, { cwd: process.cwd(), persistent: true }).on("add", function (path) {
        path = normalizePath(path);
        if (m.match(path)) {
          if (ready) {
            _this5.enqueueCopy(path);
          } else {
            firstCopyCount += 1;
            _this5.enqueueCopy(path, function () {
              firstCopyCount -= 1;
              fireReadyIfReady();
            });
          }
        }
      }).on("unlink", function (path) {
        path = normalizePath(path);
        if (m.match(path)) {
          var dstPath = _this5.src2dst(path);
          if (dstPath !== path) {
            _this5.enqueueRemove(dstPath);
          }
        }
      }).on("change", function (path) {
        path = normalizePath(path);
        if (m.match(path)) {
          _this5.enqueueCopy(path);
        }
      }).on("ready", function () {
        ready = true;
        fireReadyIfReady();
      }).on("error", function (err) {
        _this5.emit("watch-error", err);
      });
    }
  }, {
    key: "unwatch",

    /**
     * Stop watching.
     */
    value: function unwatch() {
      if (this[WATCHER] != null) {
        this[WATCHER].close();
        this[WATCHER] = null;
      }
    }
  }, {
    key: "close",

    /**
     * Stop watching.
     */
    value: function close() {
      this.unwatch();
    }
  }]);

  return Cpx;
})(_EventEmitter2.EventEmitter);

exports["default"] = Cpx;
module.exports = exports["default"];